package cn.com.qmth.examcloud.core.examwork.api.controller;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.ErrorMsg;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.CourseGroupBean;
import cn.com.qmth.examcloud.core.examwork.dao.CourseGroupRelationRepo;
import cn.com.qmth.examcloud.core.examwork.dao.CourseGroupRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.CourseGroupEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.CourseGroupRelation;
import cn.com.qmth.examcloud.core.examwork.dao.entity.Exam;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgPK;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudent;
import cn.com.qmth.examcloud.core.examwork.dao.enums.ExamType;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamService;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamStudentService;
import io.swagger.annotations.ApiOperation;

/**
 * 考试服务API Created by songyue on 17/1/13.
 */
@RestController
@RequestMapping("${app.api.root}")
public class ExamController extends ControllerSupport {

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamService examService;

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamStudentService examStudentService;

	@Autowired
	CourseGroupRepo courseGroupRepo;

	@Autowired
	ExamOrgRepo examOrgRepo;
	
	@Autowired
	CourseGroupRelationRepo courseGroupRelationRepo;

	@Autowired
	private JdbcTemplate jdbcTemplate;

	@ApiOperation(value = "查询所有考试批次", notes = "分页带查询")
	@GetMapping("/exam/all/{curPage}/{pageSize}")
	public ResponseEntity getAllExam(HttpServletRequest request, @ModelAttribute Exam examCriteria,
			@PathVariable Integer curPage, @PathVariable Integer pageSize) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			examCriteria.setOrgId(accessUser.getRootOrgId());
		} else {
			return new ResponseEntity(HttpStatus.NOT_FOUND);
		}
		return new ResponseEntity(
				examService.getAllExam(examCriteria,
						new PageRequest(curPage, pageSize, new Sort(Direction.DESC, "id"))),
				HttpStatus.OK);
	}

	@ApiOperation(value = "查询所有考试批次(包含有效)", notes = "不分页带查询")
	@GetMapping("/exam/all")
	public ResponseEntity getEnableExam(HttpServletRequest request,
			@ModelAttribute Exam examCriteria) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			examCriteria.setOrgId(accessUser.getRootOrgId());
		}
		examCriteria.setEnable(true);
		return new ResponseEntity(examService.getAllExam(examCriteria), HttpStatus.OK);
	}

	@ApiOperation(value = "根据机构ID和考试类型查询所有有效考试批次", notes = "根据机构ID和考试类型查询所有有效考试批次")
	@GetMapping("/exam/rootOrgId/{orgId}/{examType}")
	public ResponseEntity getEnableExamByRootOrgId(@PathVariable Long orgId,
			@PathVariable String examType) {
		if (orgId == null) {
			return new ResponseEntity(new ErrorMsg("机构ID不能为空"), HttpStatus.OK);
		}
		Exam examCriteria = new Exam();
		examCriteria.setOrgId(orgId);
		if (StringUtils.isNotBlank(examType)) {
			examCriteria.setExamType(ExamType.valueOf(examType));
		}
		examCriteria.setEnable(true);
		return new ResponseEntity(examService.getAllExam(examCriteria), HttpStatus.OK);
	}

	@ApiOperation(value = "查询所有考试批次(包含有效无效)", notes = "不分页带查询")
	@GetMapping("/exam/all/both")
	public ResponseEntity getAllExam(HttpServletRequest request,
			@ModelAttribute Exam examCriteria) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			examCriteria.setOrgId(accessUser.getRootOrgId());
		}
		return new ResponseEntity(examService.getAllExam(examCriteria), HttpStatus.OK);
	}

	@ApiOperation(value = "根据名称查询考试批次(包含有效)", notes = "不分页带查询")
	@GetMapping("/exam/name")
	public ResponseEntity getEnableExamByName(HttpServletRequest request,
			@ModelAttribute Exam examCriteria) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			examCriteria.setRootOrgId(accessUser.getRootOrgId());
		}
		examCriteria.setEnable(true);
		return new ResponseEntity(examService.findByNameAndRootOrgId(examCriteria.getName(),
				examCriteria.getRootOrgId()), HttpStatus.OK);
	}

	@ApiOperation(value = "按ID查询考试批次", notes = "ID查询")
	@GetMapping("/exam/{id}")
	public ResponseEntity<Exam> getExamById(@PathVariable Long id) {
		return new ResponseEntity<Exam>(examService.getExamById(id), HttpStatus.OK);
	}

	@ApiOperation(value = "新增考试批次", notes = "新增")
	@PostMapping("/exam")
	public Exam addExam(HttpServletRequest request, @RequestBody Exam exam) {

		if (null != exam.getBeforeExamRemark() && exam.getBeforeExamRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "考前说明内容过大");
		}
		if (null != exam.getAfterExamRemark() && exam.getAfterExamRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "考后说明内容过大");
		}
		if (null != exam.getCheatingRemark() && exam.getCheatingRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "作弊说明内容过大");
		}

		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			exam.setOrgId(accessUser.getOrgId());
			exam.setRootOrgId(accessUser.getRootOrgId());
			exam.setCreateTime(new Date());
			exam.setCanStuDel(true);
			if (!examService.checkExamName(exam)) {
				throw new StatusException("EXAMWORK-001001", "考试名称已存在,请重新填写");
			}
			Exam ret = examService.insertExam(exam, accessUser);
			return ret;
		} else {
			throw new StatusException("EXAMWORK-001002", "accessUser为空");
		}
	}

	@ApiOperation(value = "更新考试批次", notes = "更新")
	@PutMapping("/exam")
	public Exam updateExam(@RequestBody Exam exam) {
		if (!examService.checkExamName(exam)) {
			throw new StatusException("EXAMWORK-001001", "考试名称已存在,请重新填写");
		}

		if (null != exam.getBeforeExamRemark() && exam.getBeforeExamRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "考前说明内容过大");
		}
		if (null != exam.getAfterExamRemark() && exam.getAfterExamRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "考后说明内容过大");
		}
		if (null != exam.getCheatingRemark() && exam.getCheatingRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "作弊说明内容过大");
		}

		Exam ret = examService.saveExam(exam);
		return ret;
	}

	@ApiOperation(value = "按ID删除考试批次", notes = "删除")
	@DeleteMapping("/exam/{id}")
	public ResponseEntity deleteExam(@PathVariable String id) {
		try {
			List<Long> examIds = Stream.of(id.split(",")).map(s -> Long.parseLong(s.trim()))
					.collect(Collectors.toList());
			examService.deleteExam(examIds);
			return new ResponseEntity(HttpStatus.OK);
		} catch (Exception e) {
			log.error("删除失败:", e);
			return new ResponseEntity(new ErrorMsg(e.getMessage()),
					HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@ApiOperation(value = "启用考试", notes = "启用考试")
	@PutMapping("/exam/enable/{ids}")
	public ResponseEntity enableUser(@PathVariable String ids) {
		List<Long> examIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long examId : examIds) {
			Exam exam = examRepo.findOne(examId);
			exam.setEnable(true);
			examRepo.save(exam);
		}
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "禁用考试", notes = "禁用考试")
	@PutMapping("/exam/disable/{ids}")
	public ResponseEntity disableUser(@PathVariable String ids) {
		List<Long> examIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long examId : examIds) {
			Exam exam = examRepo.findOne(examId);
			exam.setEnable(false);
			examRepo.save(exam);
		}
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "回填考生不可删除标志", notes = "回填考生不可删除标志")
	@PutMapping("/exam/canNotDel/{id}")
	public ResponseEntity disableUser(@PathVariable Long id) {
		examRepo.canNotDel(id);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "网考初始化更新考务信息", notes = "网考初始化更新考务信息")
	@PutMapping("/exam/updateExamInfo")
	public ResponseEntity updateExamInfo(@RequestParam Long examId, @RequestParam Long examStuId) {
		examRepo.canNotDel(examId);
		examStudentService.updateExamStudentInfo(examStuId);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "网考更新考务信息", notes = "网考更新考务信息")
	@PutMapping("/exam/updateExamStudent")
	public ResponseEntity updateExamStudent(@RequestParam Long examId,
			@RequestParam Long examStuId) {
		examRepo.canNotDel(examId);
		ExamStudent examStudent = examStudentRepo.findOne(examStuId);
		examStudent.setFinished(true);
		examStudentRepo.save(examStudent);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "查询课程组集合", notes = "")
	@GetMapping("/exam/queryCourseGroupList/{examId}/{curPage}/{pageSize}")
	public List<CourseGroupBean> queryCourseGroupList(@PathVariable Long examId,
			@PathVariable Integer curPage, @PathVariable Integer pageSize) {

		Pageable pageable = new PageRequest(curPage-1, pageSize, Sort.Direction.DESC, "updateTime");
		List<CourseGroupEntity> groupList = courseGroupRepo.findAllByExamId(examId, pageable);

		List<CourseGroupBean> ret = Lists.newArrayList();

		for (CourseGroupEntity curCourseGroup : groupList) {
			CourseGroupBean bean = new CourseGroupBean();
			bean.setBeginTime(curCourseGroup.getBeginTime());
			bean.setCreationTime(curCourseGroup.getCreationTime());
			bean.setDescription(curCourseGroup.getDescription());
			bean.setEndTime(curCourseGroup.getEndTime());
			bean.setExamId(curCourseGroup.getExamId());
			bean.setId(curCourseGroup.getId());
			bean.setName(curCourseGroup.getName());
			bean.setUpdateTime(curCourseGroup.getUpdateTime());

			List<Long> courseIdList = Lists.newArrayList();
			bean.setCourseIdList(courseIdList);

			List<CourseGroupRelation> relationList = courseGroupRelationRepo
					.findAllByGroupId(bean.getId());

			for (CourseGroupRelation cur : relationList) {
				courseIdList.add(cur.getCourseId());
			}
			
			ret.add(bean);
		}

		return ret;
	}

	@ApiOperation(value = "通过课程ID查询课程组集合", notes = "")
	@GetMapping("/exam/queryCourseGroupList/{examId}/{courseId}")
	public List<CourseGroupBean> queryCourseGroupList(@PathVariable Long examId,
			@PathVariable Long courseId) {

		List<CourseGroupRelation> relationList = courseGroupRelationRepo
				.findAllByCourseIdAndExamId(courseId, examId);

		List<Long> groupIdList = Lists.newArrayList();
		for (CourseGroupRelation cur : relationList) {
			Long groupId = cur.getGroupId();
			groupIdList.add(groupId);
		}

		List<CourseGroupEntity> groupList = courseGroupRepo
				.findAllByIdInOrderByUpdateTimeDesc(groupIdList);

		List<CourseGroupBean> ret = Lists.newArrayList();

		for (CourseGroupEntity curCourseGroup : groupList) {
			CourseGroupBean bean = new CourseGroupBean();
			bean.setBeginTime(curCourseGroup.getBeginTime());
			bean.setCreationTime(curCourseGroup.getCreationTime());
			bean.setDescription(curCourseGroup.getDescription());
			bean.setEndTime(curCourseGroup.getEndTime());
			bean.setExamId(curCourseGroup.getExamId());
			bean.setId(curCourseGroup.getId());
			bean.setName(curCourseGroup.getName());
			bean.setUpdateTime(curCourseGroup.getUpdateTime());

			List<Long> courseIdList = Lists.newArrayList();
			bean.setCourseIdList(courseIdList);

			List<CourseGroupRelation> curRelationList = courseGroupRelationRepo
					.findAllByGroupId(bean.getId());

			for (CourseGroupRelation cur : curRelationList) {
				courseIdList.add(cur.getCourseId());
			}
		}

		return ret;
	}

	@ApiOperation(value = "更新课程组", notes = "")
	@PutMapping("/exam/courseGroup")
	public void updateCourseGroup(@RequestBody CourseGroupBean courseGroupBean,
			HttpServletRequest request) {

		CourseGroupEntity courseGroup = new CourseGroupEntity();
		courseGroup.setBeginTime(courseGroupBean.getBeginTime());
		courseGroup.setCreationTime(courseGroupBean.getCreationTime());
		courseGroup.setDescription(courseGroupBean.getDescription());
		courseGroup.setEndTime(courseGroupBean.getEndTime());
		courseGroup.setExamId(courseGroupBean.getExamId());
		courseGroup.setId(courseGroupBean.getId());
		courseGroup.setName(courseGroupBean.getName());
		courseGroup.setUpdateTime(courseGroupBean.getUpdateTime());

		courseGroupRepo.save(courseGroup);
		List<Long> courseIdList = courseGroupBean.getCourseIdList();

		List<CourseGroupRelation> relationList = Lists.newArrayList();

		courseGroupRelationRepo.deleteByGroupId(courseGroup.getId());
		
		for (Long cur : courseIdList) {
			CourseGroupRelation relation = new CourseGroupRelation();
			relation.setCourseId(cur);
			relation.setExamId(courseGroupBean.getExamId());
			relation.setGroupId(courseGroup.getId());
			relationList.add(relation);
		}
		courseGroupRelationRepo.save(relationList);
	}

	@ApiOperation(value = "删除课程组", notes = "")
	@DeleteMapping("/exam/courseGroup/{id}")
	public void deleteSpecificExam(@PathVariable Long id, HttpServletRequest request) {
		courseGroupRepo.delete(id);
		courseGroupRelationRepo.deleteByGroupId(id);
	}

	@ApiOperation(value = "查询考试相关的学习中心设置", notes = "")
	@GetMapping("/exam/getExamOrgList/{examId}/{curPage}/{pageSize}")
	public List<?> getExamOrgList(@PathVariable Long examId, @PathVariable Integer curPage,
			@PathVariable Integer pageSize) {
		Pageable pageable = new PageRequest(curPage-1, pageSize, Sort.Direction.DESC, "updateTime");
		List<ExamOrgEntity> allByExamId = examOrgRepo.findAllByExamId(examId, pageable);
		return allByExamId;
	}

	@ApiOperation(value = "新增考试相关的学习中心设置", notes = "")
	@PostMapping("/exam/examOrg")
	public ExamOrgEntity addExamOrg(@RequestBody ExamOrgEntity examOrg) {
		ExamOrgEntity ret = examOrgRepo.save(examOrg);
		return ret;
	}

	@ApiOperation(value = "更新考试相关的学习中心设置", notes = "")
	@PutMapping("/exam/examOrg")
	public ExamOrgEntity updateExamOrg(@RequestBody ExamOrgEntity examOrg) {
		ExamOrgEntity ret = examOrgRepo.save(examOrg);
		return ret;
	}
	
	@ApiOperation(value = "删除考试相关的学习中心设置", notes = "")
	@DeleteMapping("/exam/examOrg/{examId}/{orgId}")
	public void deleteExamOrg(@PathVariable Long examId, @PathVariable Long orgId) {
		ExamOrgEntity examOrg = new ExamOrgEntity();
		examOrg.setExamId(examId);
		examOrg.setOrgId(orgId);
		examOrgRepo.delete(examOrg);
	}

}
